@@ -70,7 +70,8 @@ sub lives_and(&;$) {
     );
 }
 
-sub live_or_die {
+util 'live_or_die';
+sub live_or_die(&) {
     my ( $code ) = @_;
     my $return = eval { $code->(); 'did not die' } || "died";
     my $msg = $@;
@@ -92,6 +93,60 @@ sub live_or_die {
 
 1;
 
+=head1 NAME
+
+Fennec::Assert::Core::Exception - Functions to test code that throws exceptions
+
+=head1 DESCRIPTION
+
+Functions to test code that throws warnings. Emulates L<Test::Exception>.
+
+=head1 SYNOPSIS
+
+    dies_ok { die( 'xxx' )} "Should die";
+    lives_ok { 1 } "Should live";
+    throws_ok { die( 'xxx' )} qr/xxx/, "Throws 'xxx'";
+    lives_and { ok( 1, "We did not die" )} "Ooops we died";
+
+=head1 FUNCTIONS
+
+=over 4
+
+=item lives_ok { ... } $name
+
+Test passes if the codeblock does not die.
+
+=item dies_ok { ... } $name
+
+Test passes if the codeblock dies.
+
+=item throws_ok { ... } qr//, $name
+
+Test passes if the codeblock dies, and the thrown message matches the regex.
+
+=item lives_and { ... } $name
+
+Does nothing if the codeblock lives, produces a failed test result if the
+codeblock dies.
+
+=back
+
+=head1 INTERNAL API
+
+=head2 FUNCTIONS
+
+=over 4
+
+=item $bool = live_or_die(sub { ... })
+
+=item ( $bool, $msg ) = live_or_die(sub { ... })
+
+Run a codeblock and check if it lives or dies. In array context will return a
+boolean, if the code died the error will also be returned. In scalar context
+only a boolean will be returned.
+
+=back
+
 =head1 AUTHORS
 
 Chad Granum L<exodist7@gmail.com>
